summaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
authorTristan Sloughter <t@crashfast.com>2014-09-17 08:52:49 -0500
committerTristan Sloughter <t@crashfast.com>2014-09-17 08:52:49 -0500
commit48769e75c224ccd8e04df250b86b50540270ce9a (patch)
treed4a094d9c82b73a9d57594b366f92fd32d6e7ff7 /src
parent7d43d0e092be4fff0496e8c3522fbd526074b37f (diff)
wip: move to erlydtl for rebar templates
Diffstat (limited to 'src')
-rw-r--r--src/rebar.app.src38
-rw-r--r--src/rebar_mustache.erl230
-rw-r--r--src/rebar_templater.erl46
3 files changed, 68 insertions, 246 deletions
diff --git a/src/rebar.app.src b/src/rebar.app.src
new file mode 100644
index 0000000..818a0c7
--- /dev/null
+++ b/src/rebar.app.src
@@ -0,0 +1,38 @@
+%% -*- mode: erlang;erlang-indent-level: 4;indent-tabs-mode: nil -*-
+%% ex: ts=4 sw=4 ft=erlang et
+
+{application, rebar,
+ [{description, "Rebar: Erlang Build Tool"},
+ {vsn, "3.0.0"},
+ {modules, []},
+ {registered, []},
+ {applications, [kernel,
+ stdlib,
+ sasl,
+ compiler,
+ crypto,
+ syntax_tools,
+ tools,
+ erlware_commons,
+ relx,
+ inets]},
+ {env, [
+ %% Default log level
+ {log_level, warn},
+
+ %% any_dir processing modules
+ {providers, [rebar_prv_escripter,
+ rebar_prv_deps,
+ rebar_prv_lock,
+ rebar_prv_install_deps,
+ rebar_prv_packages,
+ rebar_erlydtl_compiler,
+ rebar_prv_compile,
+ rebar_prv_app_discovery,
+ rebar_prv_shell,
+ rebar_prv_tar,
+ rebar_prv_new,
+ rebar_prv_update,
+ rebar_prv_release]}
+ ]}
+ ]}.
diff --git a/src/rebar_mustache.erl b/src/rebar_mustache.erl
deleted file mode 100644
index 9016c0f..0000000
--- a/src/rebar_mustache.erl
+++ /dev/null
@@ -1,230 +0,0 @@
-%% The MIT License
-%%
-%% Copyright (c) 2009 Tom Preston-Werner <tom@mojombo.com>
-%%
-%% Permission is hereby granted, free of charge, to any person obtaining a copy
-%% of this software and associated documentation files (the "Software"), to deal
-%% in the Software without restriction, including without limitation the rights
-%% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-%% copies of the Software, and to permit persons to whom the Software is
-%% furnished to do so, subject to the following conditions:
-%%
-%% The above copyright notice and this permission notice shall be included in
-%% all copies or substantial portions of the Software.
-%%
-%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-%% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-%% THE SOFTWARE.
-
-%% See the README at http://github.com/mojombo/mustache.erl for additional
-%% documentation and usage examples.
-
--module(rebar_mustache). %% v0.1.0
--author("Tom Preston-Werner").
--export([compile/1, compile/2, render/1, render/2, render/3, get/2, get/3, escape/1, start/1]).
-
--record(mstate, {mod = undefined,
- section_re = undefined,
- tag_re = undefined}).
-
--define(MUSTACHE_STR, "rebar_mustache").
-
-compile(Body) when is_list(Body) ->
- State = #mstate{},
- CompiledTemplate = pre_compile(Body, State),
- % io:format("~p~n~n", [CompiledTemplate]),
- % io:format(CompiledTemplate ++ "~n", []),
- {ok, Tokens, _} = erl_scan:string(CompiledTemplate),
- {ok, [Form]} = erl_parse:parse_exprs(Tokens),
- Bindings = erl_eval:new_bindings(),
- {value, Fun, _} = erl_eval:expr(Form, Bindings),
- Fun;
-compile(Mod) ->
- TemplatePath = template_path(Mod),
- compile(Mod, TemplatePath).
-
-compile(Mod, File) ->
- code:purge(Mod),
- {module, _} = code:load_file(Mod),
- {ok, TemplateBin} = file:read_file(File),
- Template = re:replace(TemplateBin, "\"", "\\\\\"", [global, {return,list}]),
- State = #mstate{mod = Mod},
- CompiledTemplate = pre_compile(Template, State),
- % io:format("~p~n~n", [CompiledTemplate]),
- % io:format(CompiledTemplate ++ "~n", []),
- {ok, Tokens, _} = erl_scan:string(CompiledTemplate),
- {ok, [Form]} = erl_parse:parse_exprs(Tokens),
- Bindings = erl_eval:new_bindings(),
- {value, Fun, _} = erl_eval:expr(Form, Bindings),
- Fun.
-
-render(Mod) ->
- TemplatePath = template_path(Mod),
- render(Mod, TemplatePath).
-
-render(Body, Ctx) when is_list(Body) ->
- TFun = compile(Body),
- render(undefined, TFun, Ctx);
-render(Mod, File) when is_list(File) ->
- render(Mod, File, dict:new());
-render(Mod, CompiledTemplate) ->
- render(Mod, CompiledTemplate, dict:new()).
-
-render(Mod, File, Ctx) when is_list(File) ->
- CompiledTemplate = compile(Mod, File),
- render(Mod, CompiledTemplate, Ctx);
-render(Mod, CompiledTemplate, Ctx) ->
- Ctx2 = dict:store('__mod__', Mod, Ctx),
- lists:flatten(CompiledTemplate(Ctx2)).
-
-pre_compile(T, State) ->
- SectionRE = "\{\{\#([^\}]*)}}\s*(.+?){{\/\\1\}\}\s*",
- {ok, CompiledSectionRE} = re:compile(SectionRE, [dotall]),
- TagRE = "\{\{(#|=|!|<|>|\{)?(.+?)\\1?\}\}+",
- {ok, CompiledTagRE} = re:compile(TagRE, [dotall]),
- State2 = State#mstate{section_re = CompiledSectionRE, tag_re = CompiledTagRE},
- "fun(Ctx) -> " ++
- "CFun = fun(A, B) -> A end, " ++
- compiler(T, State2) ++ " end.".
-
-compiler(T, State) ->
- Res = re:run(T, State#mstate.section_re),
- case Res of
- {match, [{M0, M1}, {N0, N1}, {C0, C1}]} ->
- Front = string:substr(T, 1, M0),
- Back = string:substr(T, M0 + M1 + 1),
- Name = string:substr(T, N0 + 1, N1),
- Content = string:substr(T, C0 + 1, C1),
- "[" ++ compile_tags(Front, State) ++
- " | [" ++ compile_section(Name, Content, State) ++
- " | [" ++ compiler(Back, State) ++ "]]]";
- nomatch ->
- compile_tags(T, State)
- end.
-
-compile_section(Name, Content, State) ->
- Mod = State#mstate.mod,
- Result = compiler(Content, State),
- "fun() -> " ++
- "case " ++ ?MUSTACHE_STR ++ ":get(" ++ Name ++ ", Ctx, " ++ atom_to_list(Mod) ++ ") of " ++
- "\"true\" -> " ++
- Result ++ "; " ++
- "\"false\" -> " ++
- "[]; " ++
- "List when is_list(List) -> " ++
- "[fun(Ctx) -> " ++ Result ++ " end(dict:merge(CFun, SubCtx, Ctx)) || SubCtx <- List]; " ++
- "Else -> " ++
- "throw({template, io_lib:format(\"Bad context for ~p: ~p\", [" ++ Name ++ ", Else])}) " ++
- "end " ++
- "end()".
-
-compile_tags(T, State) ->
- Res = re:run(T, State#mstate.tag_re),
- case Res of
- {match, [{M0, M1}, K, {C0, C1}]} ->
- Front = string:substr(T, 1, M0),
- Back = string:substr(T, M0 + M1 + 1),
- Content = string:substr(T, C0 + 1, C1),
- Kind = tag_kind(T, K),
- Result = compile_tag(Kind, Content, State),
- "[\"" ++ Front ++
- "\" | [" ++ Result ++
- " | " ++ compile_tags(Back, State) ++ "]]";
- nomatch ->
- "[\"" ++ T ++ "\"]"
- end.
-
-tag_kind(_T, {-1, 0}) ->
- none;
-tag_kind(T, {K0, K1}) ->
- string:substr(T, K0 + 1, K1).
-
-compile_tag(none, Content, State) ->
- Mod = State#mstate.mod,
- ?MUSTACHE_STR ++ ":escape(" ++ ?MUSTACHE_STR ++ ":get(" ++ Content ++ ", Ctx, " ++ atom_to_list(Mod) ++ "))";
-compile_tag("{", Content, State) ->
- Mod = State#mstate.mod,
- ?MUSTACHE_STR ++ ":get(" ++ Content ++ ", Ctx, " ++ atom_to_list(Mod) ++ ")";
-compile_tag("!", _Content, _State) ->
- "[]".
-
-template_dir(Mod) ->
- DefaultDirPath = filename:dirname(code:which(Mod)),
- case application:get_env(mustache, templates_dir) of
- {ok, DirPath} when is_list(DirPath) ->
- case filelib:ensure_dir(DirPath) of
- ok -> DirPath;
- _ -> DefaultDirPath
- end;
- _ ->
- DefaultDirPath
- end.
-template_path(Mod) ->
- DirPath = template_dir(Mod),
- Basename = atom_to_list(Mod),
- filename:join(DirPath, Basename ++ ".mustache").
-
-get(Key, Ctx) when is_list(Key) ->
- {ok, Mod} = dict:find('__mod__', Ctx),
- get(list_to_atom(Key), Ctx, Mod);
-get(Key, Ctx) ->
- {ok, Mod} = dict:find('__mod__', Ctx),
- get(Key, Ctx, Mod).
-
-get(Key, Ctx, Mod) when is_list(Key) ->
- get(list_to_atom(Key), Ctx, Mod);
-get(Key, Ctx, Mod) ->
- case dict:find(Key, Ctx) of
- {ok, Val} ->
- % io:format("From Ctx {~p, ~p}~n", [Key, Val]),
- to_s(Val);
- error ->
- case erlang:function_exported(Mod, Key, 1) of
- true ->
- Val = to_s(Mod:Key(Ctx)),
- % io:format("From Mod/1 {~p, ~p}~n", [Key, Val]),
- Val;
- false ->
- case erlang:function_exported(Mod, Key, 0) of
- true ->
- Val = to_s(Mod:Key()),
- % io:format("From Mod/0 {~p, ~p}~n", [Key, Val]),
- Val;
- false ->
- []
- end
- end
- end.
-
-to_s(Val) when is_integer(Val) ->
- integer_to_list(Val);
-to_s(Val) when is_float(Val) ->
- io_lib:format("~.2f", [Val]);
-to_s(Val) when is_atom(Val) ->
- atom_to_list(Val);
-to_s(Val) ->
- Val.
-
-escape(HTML) ->
- escape(HTML, []).
-
-escape([], Acc) ->
- lists:reverse(Acc);
-escape(["<" | Rest], Acc) ->
- escape(Rest, lists:reverse("&lt;", Acc));
-escape([">" | Rest], Acc) ->
- escape(Rest, lists:reverse("&gt;", Acc));
-escape(["&" | Rest], Acc) ->
- escape(Rest, lists:reverse("&amp;", Acc));
-escape([X | Rest], Acc) ->
- escape(Rest, [X | Acc]).
-
-%%---------------------------------------------------------------------------
-
-start([T]) ->
- Out = render(list_to_atom(T)),
- io:format(Out ++ "~n", []).
diff --git a/src/rebar_templater.erl b/src/rebar_templater.erl
index c035ec3..a795b66 100644
--- a/src/rebar_templater.erl
+++ b/src/rebar_templater.erl
@@ -37,6 +37,7 @@
-include("rebar.hrl").
-define(TEMPLATE_RE, "^[^._].*\\.template\$").
+-define(ERLYDTL_COMPILE_OPTS, [report_warnings, return_errors, {auto_escape, false}, {out_dir, false}]).
%% ===================================================================
%% Public API
@@ -80,20 +81,19 @@ resolve_variables([{Key, Value0} | Rest], Dict) when is_list(Value0) ->
Value = render(list_to_binary(Value0), Dict),
resolve_variables(Rest, dict:store(Key, Value, Dict));
resolve_variables([{Key, {list, Dicts}} | Rest], Dict) when is_list(Dicts) ->
%% just un-tag it so mustache can use it
resolve_variables(Rest, dict:store(Key, Dicts, Dict));
resolve_variables([_Pair | Rest], Dict) ->
resolve_variables(Rest, Dict).
%%
%% Render a binary to a string, using mustache and the specified context
%%
-render(Bin, Context) ->
- %% Be sure to escape any double-quotes before rendering...
- ReOpts = [global, {return, list}],
- Str0 = re:replace(Bin, "\\\\", "\\\\\\", ReOpts),
- Str1 = re:replace(Str0, "\"", "\\\\\"", ReOpts),
- rebar_mustache:render(Str1, Context).
+render(Template, Context) when is_atom(Template) ->
+ Template:render(Context);
+render(Template, Context) ->
+ Module = list_to_atom(Template++"_dtl"),
+ Module:render(Context).
%% ===================================================================
%% Internal functions
@@ -156,19 +156,19 @@ create1(State, AppDir, TemplateId) ->
%% Handle variables that possibly include other variables in their
%% definition
- Context = resolve_variables(dict:to_list(Context3), Context3),
+ %Context = resolve_variables(dict:to_list(Context3), Context3),
- ?DEBUG("Resolved Template ~p context: ~p\n",
- [TemplateId, dict:to_list(Context)]),
+ %?DEBUG("Resolved Template ~p context: ~p\n",
+ %[TemplateId, dict:to_list(Context)]),
%% Now, use our context to process the template definition -- this
%% permits us to use variables within the definition for filenames.
- FinalTemplate = consult(render(load_file(Files, Type, Template), Context)),
- ?DEBUG("Final template def ~p: ~p\n", [TemplateId, FinalTemplate]),
+ %FinalTemplate = consult(render(load_file(Files, Type, Template), Context)),
+ %?DEBUG("Final template def ~p: ~p\n", [TemplateId, FinalTemplate]),
%% Execute the instructions in the finalized template
Force = rebar_state:get(State, force, "0"),
- execute_template(Files, FinalTemplate, Type, Template, Context, Force, []).
+ execute_template([], TemplateTerms, Type, TemplateId, Context3, Force, []).
find_templates(State) ->
%% Load a list of all the files in the escript -- cache them since
@@ -377,8 +377,12 @@ execute_template(Files, [{'case', Variable, Values, Instructions} | Rest], Templ
execute_template(Files, [{template, Input, Output} | Rest], TemplateType,
TemplateName, Context, Force, ExistingFiles) ->
InputName = filename:join(filename:dirname(TemplateName), Input),
- File = load_file(Files, TemplateType, InputName),
- case write_file(Output, render(File, Context), Force) of
+ %File = load_file(Files, TemplateType, InputName),
+ OutputTemplateName = make_template_name("rebar_output_template", Output),
+ {ok, OutputTemplateName1} = erlydtl:compile_template(Output, OutputTemplateName, ?ERLYDTL_COMPILE_OPTS),
+ {ok, OutputRendered} = OutputTemplateName1:render(dict:to_list(Context)),
+ {ok, Rendered} = render(Input, dict:to_list(Context)),
+ case write_file(lists:flatten(io_lib:format("~s", [OutputRendered])), Rendered, Force) of
ok ->
execute_template(Files, Rest, TemplateType, TemplateName,
Context, Force, ExistingFiles);
@@ -449,3 +453,13 @@ execute_template(Files, [Other | Rest], TemplateType, TemplateName,
?WARN("Skipping unknown template instruction: ~p\n", [Other]),
execute_template(Files, Rest, TemplateType, TemplateName, Context,
Force, ExistingFiles).
+
+-spec make_template_name(string(), term()) -> module().
+make_template_name(Base, Value) ->
+ %% Seed so we get different values each time
+ random:seed(erlang:now()),
+ Hash = erlang:phash2(Value),
+ Ran = random:uniform(10000000),
+ erlang:list_to_atom(Base ++ "_" ++
+ erlang:integer_to_list(Hash) ++
+ "_" ++ erlang:integer_to_list(Ran)).
@@ -80,20 +81,19 @@ resolve_variables([{Key, Value0} | Rest], Dict) when is_list(Value0) ->
resolve_variables(Rest, dict:store(Key, Value, Dict));
resolve_variables([{Key, {list, Dicts}} | Rest], Dict) when is_list(Dicts) ->
- %% just un-tag it so mustache can use it
+ %% just un-tag it so erlydtl can use it
resolve_variables(Rest, dict:store(Key, Dicts, Dict));
resolve_variables([_Pair | Rest], Dict) ->
@@ -80,20 +81,19 @@ resolve_variables([{Key, Value0} | Rest], Dict) when is_list(Value0) ->
resolve_variables(Rest, Dict).
%%
-%% Render a binary to a string, using mustache and the specified context
+%% Render a binary to a string, using erlydtl and the specified context
%%